package ch.epfl.collections;

import static org.junit.Assert.*;

import org.junit.Test;

public abstract class StringListTest {
    abstract StringList newList();

    @Test
    public void testAdd() {
        StringList l = newList();
        l.add("hello");
        assertEquals("hello", l.get(0));
        assertEquals(1, l.size());
    }

    @Test(expected = IndexOutOfBoundsException.class)
    public void testGetEmpty() {
        StringList l = newList();
        l.get(0);
    }

    @Test
    public void testGet() {
        StringList l = newList();
        l.add("hello");
        l.add("world");
        assertEquals("hello", l.get(0));
        assertEquals("world", l.get(1));
    }

    @Test
    public void testSet() {
        StringList l = newList();
        l.add("hello");
        l.set(0, "world");
        assertEquals("world", l.get(0));
    }

    @Test
    public void testSize() {
        StringList l = newList();
        assertEquals(0, l.size());
        l.add("hello");
        assertEquals(1, l.size());
        l.remove(0);
        assertEquals(0, l.size());
    }

    @Test
    public void testIsEmpty() {
        StringList l = newList();
        assertTrue(l.isEmpty());
        l.add("hello");
        assertFalse(l.isEmpty());
        l.remove(0);
        assertTrue(l.isEmpty());
    }

    @Test
    public void testRemove() {
        StringList l = newList();
        l.add("one");
        l.add("two");
        l.add("three");
        l.remove(1);
        assertEquals(2, l.size());
        assertEquals("one", l.get(0));
        assertEquals("three", l.get(1));
    }

    @Test
    public void testRemoveHead(){
        StringList l = newList();
        l.add("one");
        l.add("two");
        l.add("three");
        l.remove(0);
        assertEquals(2, l.size());
        assertEquals("two", l.get(0));
        assertEquals("three", l.get(1));
    }

    @Test
    public void testIterator() {
        StringList l = newList();
        String[] a = new String[] { "one", "two", "three", "four" };
        for (String s: a)
            l.add(s);
        StringIterator it = l.iterator();
        int i = 0;
        while (it.hasNext()) {
            String e = it.next();
            assertEquals(a[i++], e);
        }
        assertEquals(i, a.length);
    }

    @Test
    public void testToString() {
        StringList l = newList();
        String[] a = new String[] { "one", "two", "three", "four" };
        for (String s: a)
            l.add(s);
        assertEquals("[one,two,three,four]", l.toString());
    }
}
